﻿using System.Windows;
using System.Windows.Controls;
using Microsoft.Xna.Framework;
using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using CtrEditor.Simulacion;
using CommunityToolkit.Mvvm.ComponentModel;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucBotella.xaml
    /// </summary>
    /// 

    public partial class osBotella : osBase, IosBase
    {
        private simBotella SimGeometria;

        // Otros datos y métodos relevantes para la simulación

        public static string NombreClase()
        {
            return "Botella";
        }
        private string nombre = NombreClase();
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        private float diametro;

        partial void OnDiametroChanged(float value)
        {
            SimGeometria?.SetDiameter(Diametro);
        }

        [ObservableProperty]
        private float mass;
        partial void OnMassChanged(float value)
        {
            SimGeometria?.SetMass(value);
        }

        public Vector2 GetCentro()
        {
            return new Vector2 (Left+Diametro/2,Top+Diametro/2);
        }

        public void SetCentro(float x, float y)
        { Left = x; Top = y; }

        public void SetCentro(Vector2 centro)
        { 
            Left = centro.X - Diametro / 2;
            Top = centro.Y - Diametro / 2; 
        }

        private void ActualizarGeometrias()
        {
            if (SimGeometria != null)
            {
                SimGeometria.SetDiameter(Diametro);
                SimGeometria.SetPosition(GetCentro());
            }
        }

        public osBotella()
        {
            Diametro = 0.10f;
            Mass = 1;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void SimulationStop()
        {
            // Se llama al detener la simulacion. Util para detener Storyboards
        }

        public override void UpdateGeometryStep()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }

        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds) { }

        public override void UpdateControl(int elapsedMilliseconds)
        {
            SetCentro(SimGeometria.Center);

            if (SimGeometria.Descartar)  // Ha sido marcada para remover
                RemoverDesdeSimulacion = true;
        }

        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();
            SimGeometria = simulationManager.AddCircle(Diametro, GetCentro(), Mass);
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager.Remove(SimGeometria);
        }

    }

    public partial class ucBotella : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucBotella()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }

        public void Resize(float width, float height) { }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle) { }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 10;
        }
    }
}
